<?php
namespace PTPay\MeoWallet\Controller\Payment;

use \Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ResponseInterface;
use Magento\Sales\Api\Data\OrderInterface;
use PTPay\MeoWallet\Helper\OrderManagement as OrderManagementHelper;
use PTPay\MeoWallet\Model\MeoWallet;
use PTPay\MeoWallet\Model\MeoWalletMultibanco;
use Magento\Framework\App\RequestInterface;
use PTPay\MeoWallet\Helper\Multibanco as MultibancoHelper;
/**
 * Class Redirect - Responsible for create the order and redirect the user to the meowallet gateway
 * The user reaches here, by the payment method javascript
 *
 *
 * @package PTPay\MeoWallet\Controller\Payment
 */
class Redirect extends Action
{
    /**
     * @var Context
     */
    private $context;

    /**
     * @var MeoWallet
     */

    private $meoWallet;
    /**
     * @var CheckoutSession
     */
    private $checkoutSession;

    /**
     * @var OrderManagementHelper
     */
    private $orderManagementHelper;

    /**
     * Construct the redirect action. Inject the necessary dependencies
     *
     * @param Context               $context
     * @param MeoWallet             $meoWallet
     * @param OrderManagementHelper $orderManagementHelper
     * @param CheckoutSession       $checkoutSession
     */
    public function __construct(
        Context $context,
        MeoWallet $meoWallet,
        OrderManagementHelper $orderManagementHelper,
        CheckoutSession $checkoutSession
    ) {
        $this->context = $context;
        $this->meoWallet = $meoWallet;
        $this->checkoutSession = $checkoutSession;
        parent::__construct($context);
        $this->orderManagementHelper = $orderManagementHelper;
    }

    /**
     * Dispatch request
     *
     * @return \Magento\Framework\Controller\ResultInterface|ResponseInterface
     * @throws \Magento\Framework\Exception\NotFoundException
     */
    public function execute()
    {
        $resultRedirect = $this->resultRedirectFactory->create();

        $order = $this->checkoutSession->getLastRealOrder();
        $chosenMethod = $this->getChosenMethod();



        // Set to the right status
        $this->orderManagementHelper->pendingPayment($order);

        // If the user as chosen multibanco, then try generate a reference
        if ($chosenMethod ==  MeoWallet::MEOWALLET_METHOD_MBWAY) {

            $mbway_phone = $this->getMbWayPhoneNumber();
            $mbway_response = $this->meoWallet->mbway()->createPayment($order,$mbway_phone);
            $this->checkoutSession->setMbWayResponse($mbway_response);
            return $this->_redirect('meowallet/payment/mbway');
        }

        if ($chosenMethod == MeoWallet::MEOWALLET_METHOD_MB) {
            $mbReference = $this->meoWallet->multibanco()->createMBReference($order);

            // Only show the reference, if the it was successfully created.
            // If not, try continue to wallet
            if ($mbReference != false) {

                $data = [

                    'createdAt'   => $mbReference->getData('created_at'),
                    'operationId' => $mbReference->getData('meowallet_operation_id'),
                    'entity'      => $mbReference->getData('entity'),
                    'reference'   => MultibancoHelper::formatReference($mbReference->getData('reference')),
                    'amount'      => MultibancoHelper::formatAmount($mbReference->getData('amount'),
                        $mbReference->getData('currency')),
                    'expires'     => MultibancoHelper::formatExpireDate($mbReference->getData('expires_at')),
                    'status'      => $mbReference->getData('status'),
                ];
                $this->orderManagementHelper->addMBToHistory($order, $mbReference,$data);
                return $this->showMBReference($order, $mbReference);
            }
        }


        // Create a new wallet procheckout and redirect the user
        $checkoutUrl = $this->meoWallet->proCheckout()->createCheckoutUrl($order, $chosenMethod);

        if (!$checkoutUrl) {
            $this->messageManager->addError(__('Was not possible pay with MeoWallet at this moment. Please select another payment method or try again later'));
            return $resultRedirect->setPath('sales/order/reorder', ['order_id' => $order->getEntityId()]);
        }

        $resultRedirect->setUrl($checkoutUrl);
        return $resultRedirect;

    }

    /**
     * Get the chosen meowallet payment method (WALLET, CC, MB, PAYPAL)
     *
     * @return string
     */
    protected function getChosenMethod()
    {
        $chosenMethod = $this->context->getRequest()->getParam('chosenMethod', MeoWallet::MEOWALLET_METHOD_WALLET);

        if (!$this->validChosenMethod($chosenMethod)) {
            $chosenMethod = MeoWallet::MEOWALLET_METHOD_WALLET;
        }

        return strval($chosenMethod);
    }

    /**
     * Validate the chosen method
     *
     * @param string $chosenMethod
     *
     * @return bool
     */
    protected function validChosenMethod($chosenMethod)
    {
        return in_array($chosenMethod, $this->meoWallet->getAllowedMethods());
    }

    /**
     * Redirect the user to the multibanco success page
     *
     * @param OrderInterface      $order
     * @param MeoWalletMultibanco $mbReference
     *
     * @return ResponseInterface
     */
    protected function showMBReference(OrderInterface $order, MeoWalletMultibanco $mbReference)
    {
        $meowalletOperationId = $mbReference->getData('meowallet_operation_id');
        return $this->_redirect('meowallet/payment/multibanco', ['id' => $meowalletOperationId]);
    }

    /**
     * @inheritDoc
     */
    public function createCsrfValidationException(
        RequestInterface $request
    ): ?InvalidRequestException {
        /** @var Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        $resultRedirect->setPath('*/*/edit');

        return new InvalidRequestException(
            $resultRedirect,
            [new Phrase('Invalid Form Key. Please refresh the page.')]
        );
    }

    /**
     * @inheritDoc
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }

    public function getMbWayPhoneNumber() {
        $mb_phone = $this->context->getRequest()->getParam('MBPhone', null);
        if ($mb_phone
            && strlen($mb_phone) == 9
            && preg_match('/^(9[1236])[0-9]{7}$/', $mb_phone, $n)) {
            $mbway_phone = $mb_phone;
        } else {
            return false;
        }
        return strval($mbway_phone)."";
    }

}
