<?php
namespace PTPay\MeoWallet\Helper;

use Magento\Framework\App\Helper\Context;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Catalog\Model\ResourceModel\Collection\AbstractCollection;
use Magento\Framework\Data\Collection;

use PTPay\MeoWallet\Logger\MeoWalletLogger;
use PTPay\MeoWallet\Model\MeoWalletMultibanco;
use PTPay\MeoWallet\Model\MeoWalletMultibancoFactory;
use PTPay\MeoWallet\SDK\Exceptions\ApiResponseException;

/**
 * Class Multibanco Handle all multibanco operations
 *
 * @package PTPay\MeoWallet\Helper
 */
class Multibanco extends AbstractHelper
{
    /**
     * @var MeoWalletMultibancoFactory
     */
    private $meoWalletMultibancoFactory;
    /**
     * @var MeoWalletLogger
     */
    private $logger;

    /**
     * Multibanco constructor.
     *
     * @param Context                    $context
     * @param EncryptorInterface         $encryptor
     * @param MeoWalletLogger            $logger
     * @param MeoWalletMultibancoFactory $meoWalletMultibancoFactory
     */
    public function __construct(
        Context $context,
        EncryptorInterface $encryptor,
        MeoWalletLogger $logger,
        MeoWalletMultibancoFactory $meoWalletMultibancoFactory
    ) {
        parent::__construct($context, $encryptor, $logger);
        $this->meoWalletMultibancoFactory = $meoWalletMultibancoFactory;
    }

    /**
     * Format the multibanco reference
     *
     * @param string $reference
     *
     * @return string
     */
    public static function formatReference($reference)
    {
        return implode(' ', str_split($reference, 3));
    }

    /**
     * Format the multibanco expiration date
     *
     * @param string $date Date in 2016-08-28T08:54:31+0000 format
     *
     * @return string Date in 2016-08-28 format
     */
    public static function formatExpireDate($date)
    {
        $expires = new \DateTime($date);
        return $expires->format('Y-m-d');
    }

    /**
     * Format the amount with the currency
     *
     * @param float  $amount
     * @param string $currency
     *
     * @return string
     */
    public static function formatAmount($amount, $currency)
    {
        return number_format($amount, 2) . " {$currency}";
    }

    /**
     * Create a multibanco reference
     *
     * @param OrderInterface $order
     *
     * @return bool|MeoWalletMultibanco
     */
    public function createMBReference($order)
    {
        try {
            $amount = $order->getGrandTotal();
            $orderId = $order->getIncrementId();
            $currency = $order->getOrderCurrencyCode() ?: 'EUR';
            $mbReference = $this->mbApi->createReference($amount, $orderId, $currency);
            return $this->saveMultibancoData($order, $mbReference);
        } catch (ApiResponseException $e) {
            $this->_logger->logApiResponseException('Multibanco could not be created on site', $e);
            return false;
        }
    }

    /**
     * Save the multibanco data
     *
     * @param OrderInterface $order
     * @param array          $mbReference
     *
     * @return MeoWalletMultibanco
     */
    protected function saveMultibancoData($order, $mbReference)
    {
        /** @var MeoWalletMultibanco $multibancoData */
        $multibancoData = $this->meoWalletMultibancoFactory->create();
        $multibancoData->setData('meowallet_operation_id', $mbReference['id']);
        $multibancoData->setData('sales_order_id', $order->getIncrementId());
        $multibancoData->setData('entity', $mbReference['mb']['entity']);
        $multibancoData->setData('reference', $mbReference['mb']['ref']);
        $multibancoData->setData('amount', $mbReference['amount']);
        $multibancoData->setData('currency', $mbReference['currency']);
        $multibancoData->setData('status', $mbReference['status']);
        $multibancoData->setData('expires_at', $mbReference['expires']);
        $multibancoData->save();

        // Reload the model to include created_at properties
        $multibancoData->load($multibancoData->getEntityId());

        return $multibancoData;
    }

    /**
     * Get the multibanco data from the id
     *
     * @param string $id
     *
     * @return bool|\Magento\Framework\DataObject
     */
    public function getMBReferencesByOperationId($id)
    {
        /** @var AbstractCollection $mbCollection */
        $mbCollection = $this->meoWalletMultibancoFactory->create()->getCollection();
        $mbData = $mbCollection->addFilter('meowallet_operation_id', $id)->getFirstItem();

        return ($mbData->isEmpty()) ? false : $mbData;
    }

    /**
     * Get the multibanco data from the order id
     *
     * @param string $id
     *
     * @return array
     */
    public function getMBReferencesByOrderId($id)
    {
        /** @var AbstractCollection $mbCollection */
        $mbCollection = $this->meoWalletMultibancoFactory->create()->getCollection();
        $mbData = $mbCollection->addFilter('sales_order_id', $id)->addOrder('created_at',
            Collection::SORT_ORDER_DESC)->getData();

        return (empty($mbData)) ? [] : $mbData;
    }

    /**
     * Set multibanco reference status
     *
     * @param int    $orderId
     * @param string $entity
     * @param string $reference
     * @param string $status
     *
     * @return bool
     */
    public function setMBStatus($orderId, $entity, $reference, $status)
    {
        /** @var AbstractCollection $mbCollection */
        $mbCollection = $this->meoWalletMultibancoFactory->create()->getCollection();
        $mbData = $mbCollection
            ->addFilter('sales_order_id', $orderId)
            ->addFilter('entity', $entity)
            ->addFilter('reference', $reference)
            ->getFirstItem();

        if ($mbData->isEmpty()) {
            return false;
        }

        $mbData->setData('status', $status);
        $mbData->save();
    }

}