<?php
namespace PTPay\MeoWallet\Helper;

use Magento\Framework\App\Helper\Context;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Sales\Api\Data\OrderStatusHistoryInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderManagementInterface;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Email\Sender\OrderCommentSender;
use Magento\Sales\Model\OrderFactory;
use PTPay\MeoWallet\Logger\MeoWalletLogger;
use PTPay\MeoWallet\Model\MeoWallet;
use PTPay\MeoWallet\Model\Order\Email\Sender\MBReferenceSender;
use Magento\Sales\Model\Order\Email\Sender\OrderSender;
use PTPay\MeoWallet\Model\MeoWalletMultibanco;

/**
 * Class OrderManagement
 *
 * @package PTPay\MeoWallet\Helper
 */
class OrderManagement extends AbstractHelper
{
    /**
     * @var OrderManagementInterface
     */
    private $orderManagement;
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;
    /**
     * @var OrderStatusHistoryInterface
     */
    private $orderStatusHistory;
    /**
     * @var OrderSender
     */
    private $orderSender;
    /**
     * @var MBReferenceSender
     */
    private $MBReferenceSender;
    /**
     * @var OrderFactory
     */
    private $orderFactory;
    /**
     * @var OrderCommentSender
     */
    private $orderCommentSender;
    /**
     * @var Multibanco
     */
    private $multibanco;

    /**
     * OrderManagement constructor.
     *
     * @param Context                  $context
     * @param EncryptorInterface       $encryptor
     * @param MeoWalletLogger          $logger
     * @param OrderManagementInterface $orderManagement
     * @param OrderRepositoryInterface $orderRepository
     * @param OrderFactory             $orderFactory
     * @param OrderSender              $orderSender
     * @param OrderCommentSender       $orderCommentSender
     * @param MBReferenceSender        $MBReferenceSender
     * @param Multibanco               $multibanco
     */
    public function __construct(
        Context $context,
        EncryptorInterface $encryptor,
        MeoWalletLogger $logger,
        OrderManagementInterface $orderManagement,
        OrderRepositoryInterface $orderRepository,
        OrderFactory $orderFactory,
        OrderSender $orderSender,
        OrderCommentSender $orderCommentSender,
        MBReferenceSender $MBReferenceSender,
        Multibanco $multibanco
    ) {
        parent::__construct($context, $encryptor, $logger);
        $this->orderManagement = $orderManagement;
        $this->orderRepository = $orderRepository;
        $this->orderSender = $orderSender;
        $this->MBReferenceSender = $MBReferenceSender;
        $this->orderFactory = $orderFactory;
        $this->orderCommentSender = $orderCommentSender;
        $this->multibanco = $multibanco;
    }

    /**
     * Add multibanco data to order history
     *
     * @param OrderInterface      $order
     * @param MeoWalletMultibanco $meoWalletMultibanco
     * @param array               $formattedData
     *
     * @return OrderInterface
     */
    public function addMBToHistory($order, $meoWalletMultibanco, $formattedData)
    {
        $comment = __("
        Entity: {$formattedData['entity']}
        Reference: {$formattedData['reference']}
        Amount: {$formattedData['amount']}
        Expires: {$formattedData['expires']}
        ");

        $history = $order->addStatusHistoryComment($comment, $order->getStatus());
        $history->setIsVisibleOnFront(true);
        $history->setIsCustomerNotified(true);
        $history->save();
        //DONE - item 5 - mudar o $notify (true) para depois do $multibanco pois o multi é required e o true nao
        $result = $this->MBReferenceSender->send($order, $meoWalletMultibanco,  true);
    }

    /**
     * @param OrderInterface $order
     *
     * @return bool
     */
    public function sendOrderConfirmation($order)
    {
        return $this->orderSender->send($order);
    }

    /**
     * Process the order callback
     *
     * @param $operationDetails
     *
     * @return bool
     */
    public function processCallback($operationDetails)
    {
        $memory_limit = ini_get('memory_limit');
        ini_set("memory_limit","2G");


        if ($operationDetails->event == "CREATED") {
            return true;
        }


        if (!$this->checkoutApi->verify($operationDetails)) {
            // callback is not valid
            $this->_logger->warning('Suspicious callback details received', ['Data_Received' => $operationDetails]);
            return false;
        }

        /** @var OrderInterface $order */
        $order = $this->orderFactory->create()->load($operationDetails->ext_invoiceid);

        if (!$order) {
            // Cannot load order
            $this->_logger->warning('Cannot load order by ext_invoiceid',
                ['ext_invoiceid' => $operationDetails->ext_invoiceid]);
            return false;
        }

        $meowalletStatus = $operationDetails->operation_status;
        //DONE - mudar função para ter o evento e o status comparados e ambos quando forem completed podem avançar
        $meowalletEventStatus = $operationDetails->event;
        
        //$this->_logger->error(json_encode($operationDetails));
        $this->_logger->error("MEOWallet payment operation status: $meowalletStatus - Event Status: $meowalletEventStatus - DevTest");
    
        if($meowalletStatus === 'COMPLETED' && $meowalletEventStatus === 'COMPLETED')
        {
            $this->approvePayment($order);
        }
        else {
             switch ($meowalletStatus) {
                case 'FAIL':
                    $this->failPayment($order);
                    break;
                case 'PENDING':
                    $this->pendingOrder($order);
                    break;
                default:
                    $this->_logger->error("MEOWallet payment operation status $meowalletStatus not handled by this module");
                    return false;
            }
        }
       

        if ($operationDetails->method == MeoWallet::MEOWALLET_METHOD_MB){
            $this->multibanco->setMBStatus(
                $order->getIncrementId(),
                 $operationDetails->mb_entity ,
                 $operationDetails->mb_ref,
                  $meowalletStatus);
        }
        ini_set("memory_limit",$memory_limit);
        return true;
    }

    /**
     * Complete the order. Make it paid
     *
     * @param OrderInterface $order
     */
    public function approvePayment($order)
    {
        $status = $this->getConfigData('payment/wallet/complete_order_status');
        if (!$this->hasSameStatus($order, $status)) {
            $order->setState($status)->setStatus($status);
            $this->orderRepository->save($order);
            $this->orderCommentSender->send($order, true, __('Payment received from MeoWallet'));
        }

    }

    /**
     * Check if an order has the same status
     *
     * @param OrderInterface $order
     * @param string         $status
     *
     * @return bool
     */
    public function hasSameStatus($order, $status)
    {
        return $order->getStatus() == $status;
    }

    /**
     * The payment has been failed
     *
     * @param OrderInterface $order
     */
    public function failPayment($order)
    {
        $status = $this->getConfigData('payment/wallet/cancel_order_status');
        if (!$this->hasSameStatus($order, $status)) {
            $order->setState($status)->setStatus($status);
            $this->orderRepository->save($order);
            $this->orderCommentSender->send($order, true, __('Payment failed. Order has been canceled'));
        }
    }

    /**
     * Put order on a pending state
     *
     * @param OrderInterface $order
     */
    public function pendingOrder($order)
    {
        if (!$this->hasSameStatus($order, $status)) {
            $status = $this->getConfigData('payment/wallet/new_order_status');
            $order->setState($status)->setStatus($status)->save();
        }
    }

    /**
     * Cancel the order
     *
     * @param OrderInterface $order
     */
    public function cancelOrder($order)
    {
        $status = $this->getConfigData('payment/wallet/cancel_order_status');
        if (!$this->hasSameStatus($order, $status)) {
            $order->setState($status)->setStatus($status);
            $this->orderRepository->save($order);
            $this->orderCommentSender->send($order, true, __('Order has been canceled'));
        }
    }

    /**
     * Change the order to pending payment status
     *
     * @param OrderInterface $order
     *
     * @return OrderInterface
     */
    public function pendingPayment($order)
    {
        $order->setState(Order::STATE_PENDING_PAYMENT);
        return $this->orderRepository->save($order);
    }
}
