<?php
namespace PTPay\MeoWallet\Helper;

use Magento\Catalog\Model\ResourceModel\Collection\AbstractCollection;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Framework\UrlInterface;

use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\Data\OrderItemInterface;

use Magento\Sales\Model\OrderFactory;
use PTPay\MeoWallet\Logger\MeoWalletLogger;
use PTPay\MeoWallet\Model\MeoWalletCheckout;
use PTPay\MeoWallet\Model\MeoWalletCheckoutFactory;
use PTPay\MeoWallet\SDK\Exceptions\ApiResponseException;
use PTPay\MeoWallet\SDK\Structures\Address;
use PTPay\MeoWallet\SDK\Structures\Client;
use PTPay\MeoWallet\SDK\Structures\Item;

/**
 * Operations envolving the meowallet checkout
 *
 * @package PTPay\MeoWallet\Helper
 */
class ProCheckout extends AbstractHelper
{
    /**
     * @var MeoWalletCheckoutFactory
     */
    private $meoWalletCheckoutFactory;
    /**
     * @var UrlInterface
     */
    private $urlBuilder;
    /**
     * @var OrderFactory
     */
    private $orderFactory;

    /**
     * ProCheckout constructor.
     *
     * @param MeoWalletCheckoutFactory $meoWalletCheckoutFactory
     * @param Context                  $context
     * @param MeoWalletLogger          $logger
     * @param OrderFactory             $orderFactory
     * @param EncryptorInterface       $encryptor
     *
     * @internal param UrlInterface $urlBuilder
     */
    public function __construct(
        MeoWalletCheckoutFactory $meoWalletCheckoutFactory,
        Context $context,
        MeoWalletLogger $logger,
        OrderFactory $orderFactory,
        EncryptorInterface $encryptor
    ) {
        parent::__construct($context, $encryptor, $logger);
        $this->meoWalletCheckoutFactory = $meoWalletCheckoutFactory;
        $this->orderFactory = $orderFactory;
    }

    /**
     * Create the checkout url
     *
     * @param OrderInterface $order
     * @param string         $chosenMethod
     *
     * @return bool|\Magento\Framework\Controller\Result\Redirect
     */
    public function createCheckoutUrl($order, $chosenMethod)
    {
        $client = $this->createClient($order);
        $items = $this->createItems($order);
        $amount = $order->getGrandTotal();
        $currency = $order->getBaseCurrencyCode();
        try {
            $checkoutData = $this->checkoutApi->create(
                $client,
                $items,
                $amount,
                $currency,
                $this->_urlBuilder->getUrl('meowallet/payment/success'),
                $this->_urlBuilder->getUrl('meowallet/payment/cancel'),
                $order->getCustomerId(),
                $order->getIncrementId(),
                $order->getCustomerEmail(),
                $chosenMethod
            );

            $this->saveMeowalletCheckoutData($order, $checkoutData);

            // Get the store locale
            $locale = $this->getUserLocale();

            return $checkoutData['url_redirect'] . '?locale=' . $locale;

        } catch (ApiResponseException $e) {
            $this->_logger->logApiResponseException('Could not create pro checkout', $e);
            return false;
        }
    }

    /**
     * Create the client data to send to Meowallet API
     *
     * @param OrderInterface $order
     *
     * @return Client
     */
    protected function createClient($order)
    {
        return static::_createClient($order);
    }
    public static function _createClient($order) {
        $client = new Client();
        $client->setName($order->getCustomerFirstname() . ' ' . $order->getCustomerLastname());
        $client->setEmail($order->getCustomerEmail());
        $address = new Address();
        $address->setAddress(implode(' ', $order->getBillingAddress()->getStreet()));
        $address->setCountry($order->getBillingAddress()->getCountryId());
        $address->setPostalcode($order->getBillingAddress()->getPostcode());
        $address->setCity($order->getBillingAddress()->getCity());
        $client->setAddress($address);
        return $client;
    }

    /**
     * Create the order items data do meowallet api
     *
     * @param OrderInterface $order
     *
     * @return array
     */
    protected function createItems($order)
    {
        return static::_createItems($order);
    }
    public static function _createItems($order){
        $items = [];
        foreach ($order->getItems() as $item) {
            /** @var OrderItemInterface $item */
            $meowalletItem = new Item();
            $meowalletItem->setName($item->getName());
            $meowalletItem->setQuantity($item->getQtyOrdered());
            $meowalletItem->setDescription($item->getName());
            $meowalletItem->setRef($item->getItemId());
            $items[] = $meowalletItem;
        }
        return $items;
    }

    /**
     * Save the meowallet checkout data into the database (table meowallet_checkout)
     *
     * @param OrderInterface $order
     * @param array          $checkoutData
     */
    protected function saveMeowalletCheckoutData($order, $checkoutData)
    {
        /** @var MeoWalletCheckout $meoWalletCheckout */
        $meoWalletCheckout = $this->meoWalletCheckoutFactory->create();
        $meoWalletCheckout->setData('meowallet_checkout_id', $checkoutData['id']);
        //DONE - item 9 -  mudar o de baixo para isto - $meoWalletCheckout->setData('sales_order_id', $order->getEntityId());
        // $meoWalletCheckout->setData('sales_order_id', $order->getIncrementId());
        $meoWalletCheckout->setData('sales_order_id', $order->getEntityId());
        $meoWalletCheckout->save();
    }

    /**
     * Get the current locale
     *
     * @return null|string
     */
    protected function getUserLocale()
    {
        /** @var \Magento\Framework\ObjectManagerInterface $om */
        $om = ObjectManager::getInstance();
        /** @var \Magento\Framework\Locale\Resolver $resolver */
        $resolver = $om->get('Magento\Framework\Locale\Resolver');
        return $resolver->getLocale();
    }

    /**
     * Load an order by it's checkout id
     *
     * @param string $checkoutId
     *
     * @return OrderInterface|bool
     */
    public function getOrderByCheckoutId($checkoutId)
    {
        /** @var AbstractCollection $mbCollection */
        $checkoutCollection = $this->meoWalletCheckoutFactory->create()->getCollection();
        $checkout = $checkoutCollection->addFilter('meowallet_checkout_id', $checkoutId)->getFirstItem();
        if ($checkout->isEmpty()) {
            return false;
        }
        $orderId = $checkout->getData('sales_order_id');
        $order = $this->orderFactory->create()->load($orderId);

        return ($order->isEmpty()) ? false : $order;
    }


}
