<?php
namespace PTPay\MeoWallet\Model;

use Magento\Backend\Model\UrlInterface;
use Magento\Framework\Api\AttributeValueFactory;
use Magento\Framework\Api\ExtensionAttributesFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Magento\Payment\Helper\Data;
use Magento\Payment\Model\InfoInterface;
use Magento\Payment\Model\Method\AbstractMethod;
use Magento\Payment\Model\Method\Logger;
use PTPay\MeoWallet\Helper\Mbway;
use PTPay\MeoWallet\Helper\Multibanco;
use PTPay\MeoWallet\Helper\OrderManagement;
use PTPay\MeoWallet\Helper\ProCheckout;

/**
 * Pay In Store payment method model
 */
class MeoWallet extends AbstractMethod
{

    /**
     * Meowallet specific methods
     */
    const MEOWALLET_METHOD_WALLET = 'WALLET';
    const MEOWALLET_METHOD_CC = 'CC';
    const MEOWALLET_METHOD_MB = 'MB';
    const MEOWALLET_METHOD_PAYPAL = 'PAYPAL';
    const MEOWALLET_METHOD_GPAY = 'GPAY';
    const MEOWALLET_METHOD_APAY = 'APAY';
    const MEOWALLET_METHOD_MBWAY = 'MBWAY';

    /**
     * Meowallet callback operation statuses
     */
    const OP_STATUS_COMPLETED = 'COMPLETED';
    const OP_STATUS_FAIL = 'FAIL';
    const OP_STATUS_PENDING = 'PENDING';


    /**
     * Payment code
     *
     * @var string
     */
    protected $_code = 'wallet';
    protected $_isGateway = true;
    protected $_canCapture = true;
    protected $_canCapturePartial = true;
    protected $_canOrder = true;
    protected $_isInitializeNeeded = true;

    /**
     * @var Multibanco
     */
    private $multibanco;
    /**
     * @var ProCheckout
     */
    private $proCheckout;
    /**
     * @var UrlInterface
     */
    private $urlBuilder;
    /**
     * @var OrderManagement
     */
    private $orderManagement;

    /**
     * MeoWallet constructor.
     *
     * @param Multibanco                 $multibanco
     * @param Mbway                      $mbway
     * @param ProCheckout                $proCheckout
     * @param OrderManagement            $orderManagement
     * @param UrlInterface               $urlBuilder
     * @param Context                    $context
     * @param Registry                   $registry
     * @param ExtensionAttributesFactory $extensionFactory
     * @param AttributeValueFactory      $customAttributeFactory
     * @param Data                       $paymentData
     * @param ScopeConfigInterface       $scopeConfig
     * @param Logger                     $logger
     * @param AbstractResource|null      $resource
     * @param AbstractDb|null            $resourceCollection
     * @param array                      $data
     */
    public function __construct(
        Multibanco $multibanco,
        Mbway $mbway,
        ProCheckout $proCheckout,
        OrderManagement $orderManagement,
        UrlInterface $urlBuilder,
        Context $context,
        Registry $registry,
        ExtensionAttributesFactory $extensionFactory,
        AttributeValueFactory $customAttributeFactory,
        Data $paymentData,
        ScopeConfigInterface $scopeConfig,
        Logger $logger,
        AbstractResource $resource = null,
        AbstractDb $resourceCollection = null,

        array $data = []
    ) {
        parent::__construct($context, $registry, $extensionFactory, $customAttributeFactory, $paymentData, $scopeConfig,
            $logger, $resource, $resourceCollection, $data);
        $this->multibanco = $multibanco;
        $this->proCheckout = $proCheckout;
        $this->mbway = $mbway;
        $this->urlBuilder = $urlBuilder;
        $this->orderManagement = $orderManagement;
    }

    /**
     * Only support EUR
     *
     * @param string $currencyCode
     *
     * @return bool
     */
    public function canUseForCurrency($currencyCode)
    {
        return $currencyCode == 'EUR';
    }

    /**
     * Capture payment
     *
     * @param \Magento\Framework\DataObject|InfoInterface $payment
     * @param float                                       $amount
     *
     * @return $this
     * @throws \Magento\Framework\Validator\Exception
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     */
    public function authorize(InfoInterface $payment, $amount)
    {
        return $this;
    }

    /**
     * Get the action url to where the user is redirected after place order button click
     *
     * @return string
     */
    public function getActionRedirectUrl()
    {
        return $this->urlBuilder->getUrl('meowallet/payment/redirect', ['_secure' => true]);
    }

    /**
     * Return an array with the allwed methods
     *
     * @return array
     */
    public function getAllowedMethods()
    {
        $allowedMethods = [];
        if ($this->allowWalletMethod()) {
            $allowedMethods[] = self::MEOWALLET_METHOD_WALLET;
        }
        if ($this->allowCreditCardMethod()) {
            $allowedMethods[] = self::MEOWALLET_METHOD_CC;
        }
        if ($this->allowMultibancoMethod()) {
            $allowedMethods[] = self::MEOWALLET_METHOD_MB;
        }
        if ($this->allowPaypalMethod()) {
            $allowedMethods[] = self::MEOWALLET_METHOD_PAYPAL;
        }
        if ($this->allowGooglePayMethod()) {
            $allowedMethods[] = self::MEOWALLET_METHOD_GPAY;
        }
        if ($this->allowApplePayMethod()) {
            $allowedMethods[] = self::MEOWALLET_METHOD_APAY;
        }
        if ($this->allowMbWayMethod()) {
            $allowedMethods[] = self::MEOWALLET_METHOD_MBWAY;
        }
        return $allowedMethods;
    }


    /**
     * Check if the credit card is allowed
     *
     * @return bool
     */
    protected function allowCreditCardMethod()
    {
        return $this->getConfigData('allow_cc') == 1;
    }
    /**
     * Check if the Wallet is allowed
     *
     * @return bool
     */
    protected function allowWalletMethod()
    {
        return $this->getConfigData('allow_wallet') == 1;
    }

    /**
     * Check if the multibanco is allowed
     *
     * @return bool
     */
    protected function allowMultibancoMethod()
    {
        return $this->getConfigData('allow_mb') == 1;
    }

    /**
     * Check if the paypal is allowed
     *
     * @return bool
     */
    protected function allowPaypalMethod()
    {
        return $this->getConfigData('allow_paypal') == 1;
    }

    /**
     * Check if the paypal is allowed
     *
     * @return bool
     */
    protected function allowMbWayMethod()
    {
        return $this->getConfigData('allow_mbway') == 1;
    }

    /**
     * Check if the Google Pay is allowed
     *
     * @return bool
     */
    protected function allowGooglePayMethod()
    {
        return $this->getConfigData('allow_gpay') == 1;
    }

    /**
     * Check if the Apple Pay is allowed
     *
     * @return bool
     */
    protected function allowApplePayMethod()
    {
        return $this->getConfigData('allow_apay') == 1;
    }


    /**
     * Allowed Methods
     */

    /**
     * Handle the callback from meowallet
     *
     * @param array $operationDetails
     *
     * @return bool
     */
    public function processCallback($operationDetails)
    {
        return $this->orderManagement()->processCallback($operationDetails);
    }

    /**
     * Get the order management
     *
     * @return OrderManagement
     */
    public function orderManagement()
    {
        return $this->orderManagement;
    }

    /**
     * Return the multibanco helper
     *
     * @return Multibanco
     */
    public function multibanco()
    {
        return $this->multibanco;
    }

    /**
     * Return the Procheckout helper
     *
     * @return ProCheckout
     */
    public function proCheckout()
    {
        return $this->proCheckout;
    }

    /**
     * Return the MBWay helper
     *
     * @return Mbway
     */
    public function mbway()
    {
        return $this->mbway;
    }

}
