<?php

namespace PTPay\MeoWallet\SDK;

//DONE - definir logger
use Psr\Log\LoggerInterface; 

use PTPay\MeoWallet\SDK\Exceptions\ApiResponseException;
use PTPay\MeoWallet\Logger\MeoWalletLogger;
/**
 * Class Api Handles the Meowallet api communication
 * @package PTPay\MeoWallet\SDK\
 */
class Api
{
    //DONE - definir logger
    private $logger; 
    /**
     * Sandbox environment
     */
    const ENV_SANDBOX = 'sandbox';

    /**
     * Sandbox base api url
     */
    const ENV_SANDBOX_API_URL = 'https://services.sandbox.meowallet.pt/api/v2';

    /**
     * Production environment
     */
    const ENV_PRODUCTION = 'production';

    /**
     * Production base api url
     */
    const ENV_PRODUCTION_API_URL = 'https://services.wallet.pt/api/v2';


    protected $apiKey;
    protected $environment;

    /**
     * Api constructor.
     *
     * @param string $apiKey The apikey in use
     * @param string $environment The environment in use
     * @param MeoWalletLogger $logger
     * @throws \InvalidArgumentException
     */

     //DONE - definir logger interface
    public function __construct($apiKey, $environment = 'sandbox',  MeoWalletLogger $logger = null)
    {

        $this->validateEnvironment($environment);
        $this->validateApiKey($apiKey);

        $this->apiKey = $apiKey;
        $this->environment = $environment;
        //DONE - definir logger
        $this->logger = $logger;
    }

    /**
     * Check if the environment in use is production
     * @return bool
     */
    public function isProduction()
    {
        return $this->environment === self::ENV_PRODUCTION;
    }

    /**
     * Check if the environment in use is sandbox
     * @return bool
     */
    public function isSandbox()
    {
        return $this->environment === self::ENV_SANDBOX;
    }

    /**
     * @param string $httpMethod Can be GET or POST
     * @param string $resource
     * @param null|array $requestData
     * @return ApiResponse
     * @throws ApiResponseException
     */
    public function call($httpMethod, $resource, $requestData = null)
    {

        $endpoint = $this->makeEndpoint($resource);

        $headers = array(
            'Authorization: WalletPT ' . $this->apiKey,
            'Content-Type: application/json'
        );

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_URL, $endpoint);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $httpMethod);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        if (strtoupper($httpMethod) == 'POST') {
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($requestData));
        }

        $responseData = curl_exec($ch);
        $metadata = curl_getinfo($ch);
        curl_close($ch);
        //DONE - logger a funcionar
        $response = new ApiResponse($metadata, $responseData);
        if (!$response->isSuccess()) {
            $this->logger->info('DevLog: Sem sucesso na função call');
            $this->logger->error(json_encode($response));
            throw new ApiResponseException($response);
        }
        

        return $response;
    }

    /**
     * Validate the environment
     * @param string $environment
     * @throws \InvalidArgumentException
     */
    protected function validateEnvironment($environment)
    {
        if (!in_array($environment, [self::ENV_SANDBOX, self::ENV_PRODUCTION])) {
            throw new \InvalidArgumentException('Invalid environment');
        }
        return;
    }

    /**
     * Validates the apikey
     * @param string $apiKey
     * @throws \InvalidArgumentException
     */
    protected function validateApiKey($apiKey)
    {
        if (empty($apiKey)) {
            throw new \InvalidArgumentException('ApiKey cannot be empty');
        }
    }

    /**
     * Make the api endpoint, based on what environment is chosen
     * @param string $resource
     * @return string
     */
    private function makeEndpoint($resource)
    {
        $url = null;
        if ($this->isSandbox()) {
            $url = self::ENV_SANDBOX_API_URL;
        } else {
            $url = self::ENV_PRODUCTION_API_URL;
        }
        return $url . $resource;
    }
}
